searchData={"content_type":"text/markdown","items":[{"doc":"Main functions to interact with clients.\n\nUsually you will not need to call any function here directly\nbut instead use them through a module that uses `Klife.Client`.","ref":"Klife.html","title":"Klife","type":"module"},{"doc":"","ref":"Klife.TestUtils.AsyncProducerBenchmark.html","title":"Klife.TestUtils.AsyncProducerBenchmark","type":"module"},{"doc":"","ref":"Klife.TestUtils.AsyncProducerBenchmark.html#run/2","title":"Klife.TestUtils.AsyncProducerBenchmark.run/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Benchmark.html","title":"mix benchmark","type":"task"},{"doc":"","ref":"Mix.Tasks.Benchmark.html#do_run_bench/2","title":"Mix.Tasks.Benchmark.do_run_bench/2","type":"function"},{"doc":"","ref":"Mix.Tasks.Benchmark.html#do_run_bench/3","title":"Mix.Tasks.Benchmark.do_run_bench/3","type":"function"},{"doc":"","ref":"Mix.Tasks.Benchmark.html#run/1","title":"Mix.Tasks.Benchmark.run/1","type":"function"},{"doc":"Defines a kafka client.\n\nTo use it you must do 3 steps:\n- Use it in a module\n- Config the module on your config file\n- Start the module on your supervision tree","ref":"Klife.Client.html","title":"Klife.Client","type":"behaviour"},{"doc":"When used it expects an `:otp_app` option that is the OTP application that has the client configuration.\n\n```elixir\ndefmodule MyApp.MyClient do\n  use Klife.Client, otp_app: :my_app\nend\n```\n\n> #### `use Klife.Client` {: .info}\n>\n> When you `use Klife.Client`, it will extend your module in two ways:\n>\n> - Define it as a proxy to a subset of the functions on `Klife` module,\n> using it's module's name as the `client_name` parameter.\n> One example of this is the `MyClient.produce/2` that forwards\n> both arguments to `Klife.produce/3` and inject `MyClient` as the\n> second argument.\n>\n> - Define it as a supervisor by calling `use Supervisor` and implementing\n> some related functions such as `start_link/1` and `init/1`, so it can be\n> started under on your app supervision tree.","ref":"Klife.Client.html#module-using-it-in-a-module","title":"Using it in a module - Klife.Client","type":"behaviour"},{"doc":"The client has a bunch of configuration options, you can read more below.\nBut it will look somehting like this:\n\n```elixir\nconfig :my_app, MyApp.MyClient,\n  connection: [\n    bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n    ssl: false\n  ],\n  producers: [\n    [\n      name: :my_custom_producer,\n      linger_ms: 5,\n      max_in_flight_requests: 10\n    ]\n  ],\n  topics: [\n    [name: \"my_topic_0\", producer: :my_custom_producer]\n  ]\n\n```\n\nYou can see more configuration examples on the [\"Client configuration examples\"](guides/examples/client_configuration.md) section\nor an working application example on the `example` folder on the project's repository.","ref":"Klife.Client.html#module-configuration","title":"Configuration - Klife.Client","type":"behaviour"},{"doc":"* `:connection` (non-empty `t:keyword/0`) - Required.\n\n  * `:bootstrap_servers` (list of `t:String.t/0`) - Required. List of servers to establish the initial connection. (eg: [\"localhost:9092\", \"localhost:9093\"])\n\n  * `:ssl` (`t:boolean/0`) - Specify the underlying socket module. Use `:ssl` if true and `:gen_tcp` if false. The default value is `false`.\n\n  * `:connect_opts` (list of `t:term/0`) - Options used to configure the socket connection, which are forwarded to the `connect/3` function of the underlying socket module (see ssl option above.). The default value is `[inet_backend: :socket, active: false]`.\n\n  * `:socket_opts` (list of `t:term/0`) - Options used to configure the open socket, which are forwarded to the `setopts/2` function of the underlying socket module `:inet` for `:gen_tcp` and `:ssl` for `:ssl` (see ssl option above.). The default value is `[keepalive: true]`.\n\n  * `:sasl_opts` (list of `t:term/0`) - Options to configure SASL authentication, see SASL section for supported mechanisms and examples. The default value is `[]`.\n\n* `:default_producer` (`t:atom/0`) - Name of the producer to be used on produce API calls when a specific producer is not provided via configuration or option. If not provided a default producer will be started automatically. The default value is `:klife_default_producer`.\n\n* `:default_partitioner` (`t:atom/0`) - Partitioner module to be used on produce API calls when a specific partitioner is not provided via configuration or option. The default value is `Klife.Producer.DefaultPartitioner`.\n\n* `:default_txn_pool` (`t:atom/0`) - Name of the txn pool to be used on transactions when a `:pool_name` is not provided as an option. If not provided a default txn pool will be started automatically. The default value is `:klife_default_txn_pool`.\n\n* `:txn_pools` (List of `Klife.TxnProducerPool` configurations) - List of configurations, each starting a pool of transactional producers for use with transactional api. The default value is `[]`.\n\n* `:producers` (List of `Klife.Producer` configurations) - List of configurations, each starting a new producer for use with produce api. The default value is `[]`.\n\n* `:topics` (List of `Klife.Topic` configurations) - List of topics that may have special configurations The default value is `[]`.","ref":"Klife.Client.html#module-configuration-options","title":"Configuration options: - Klife.Client","type":"behaviour"},{"doc":"Finally, it must be started on your application. It will look something like this:\n\n```elixir\ndefmodule MyApp.Application do\n  def start(_type, _args) do\n    children = [\n      # some other modules...,\n      MyApp.MyClient\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"Klife.Client.html#module-starting-it","title":"Starting it - Klife.Client","type":"behaviour"},{"doc":"In order to interact with the producer API you will work with `Klife.Record` module\nas your main input and output data structure.\n\nUsually you will give an record to some producer API function and it will return an\nenriched record with some new attributes based on what happened.\n\nSo in summary the interaction goes like this:\n- Build one or more `Klife.Record`\n- Pass it to some producer API function\n- Receive an enriched version of the provided records\n\n```elixir\nrec = %Klife.Record{value: \"some_val\", topic: \"my_topic_1\"}\n{:ok, %Klife.Record{offset: offset, partition: partition}} = MyClient.produce(rec)\n```","ref":"Klife.Client.html#module-producer-api-overview","title":"Producer API overview - Klife.Client","type":"behaviour"},{"doc":"Produce a single record.\n\nIt expects a `Klife.Record` struct containg at least `:value` and `:topic` and returns\nan ok/error tuple along side with the enriched version of the input record as described\nin [\"Producer API Overview\"](m:Klife.Client#producer-api-overview).","ref":"Klife.Client.html#c:produce/2","title":"Klife.Client.produce/2","type":"callback"},{"doc":"* `:producer` (`t:atom/0`) - Producer's name that will override the `default_producer` configuration. Ignored inside transactions.\n\n* `:partitioner` (`t:atom/0`) - Module that will override `default_partitioner` configuration.","ref":"Klife.Client.html#c:produce/2-options","title":"Options - Klife.Client.produce/2","type":"callback"},{"doc":"iex> rec = %Klife.Record{value: \"my_val\", topic: \"my_topic_1\"}\n    iex> {:ok, %Klife.Record{} = enriched_rec} = MyClient.produce(rec)\n    iex> true = is_number(enriched_rec.offset)\n    iex> true = is_number(enriched_rec.partition)","ref":"Klife.Client.html#c:produce/2-examples","title":"Examples - Klife.Client.produce/2","type":"callback"},{"doc":"Produce a single record asynchronoulsy.\n\nThe same as [`produce/2`](c:produce/2) but returns immediately. Accepts a callback\noption to execute arbitrary code after response is obtained.\n\n> #### Semantics and guarantees {: .info}\n>\n> This functions executes the callback using `Task.start/1`.\n> Therefore there is no guarantees about record delivery or callback execution.","ref":"Klife.Client.html#c:produce_async/2","title":"Klife.Client.produce_async/2","type":"callback"},{"doc":"* `:producer` (`t:atom/0`) - Producer's name that will override the `default_producer` configuration. Ignored inside transactions.\n\n* `:partitioner` (`t:atom/0`) - Module that will override `default_partitioner` configuration.\n\n* `:callback` (`t:term/0`) - MFA or function/1 that will be called with the produce result. The result is injected as the first argument on MFA and is the only argument for anonymous functions","ref":"Klife.Client.html#c:produce_async/2-options","title":"Options - Klife.Client.produce_async/2","type":"callback"},{"doc":"Anonymous Function:\n\n    iex> rec = %Klife.Record{value: \"my_val\", topic: \"my_topic_1\"}\n    iex> callback = fn resp ->\n    ...>    {:ok, enriched_rec} = resp\n    ...>    true = is_number(enriched_rec.offset)\n    ...>    true = is_number(enriched_rec.partition)\n    ...> end\n    iex> :ok = MyClient.produce_async(rec, callback: callback)\n\n    Using MFA:\n\n    iex> defmodule CB do\n    ...>    def exec(resp, my_arg1, my_arg2) do\n    ...>      \"my_arg1\" = my_arg1\n    ...>      \"my_arg2\" = my_arg2\n    ...>      {:ok, enriched_rec} = resp\n    ...>      true = is_number(enriched_rec.offset)\n    ...>      true = is_number(enriched_rec.partition)\n    ...>    end\n    ...> end\n    iex> rec = %Klife.Record{value: \"my_val\", topic: \"my_topic_1\"}\n    iex> :ok = MyClient.produce_async(rec, callback: {CB, :exec, [\"my_arg1\", \"my_arg2\"]})","ref":"Klife.Client.html#c:produce_async/2-examples","title":"Examples - Klife.Client.produce_async/2","type":"callback"},{"doc":"Produce a batch of records.\n\nIt expects a list of `Klife.Record` structs containg at least `:value` and `:topic` and returns\na list of ok/error tuples along side with the enriched version of the input record as described\nin [\"Producer API Overview\"](m:Klife.Client#producer-api-overview).\n\nThe order of the response tuples on the returning list is the same as the input list. That means\nthe first response tuple will be related to the first record on the input and so on.\n\n\n> #### Semantics and guarantees {: .info}\n>\n> This functions is semantically equivalent to call [`produce/2`](c:produce/2) multiple times and\n> wait for all responses. Which means that 2 records sent on the same batch may succeed or\n> fail independently.\n>\n> In other words that is no atomicity guarentees. If you need it see [`produce_batch_txn/2`](c:produce_batch_txn/2).\n>\n> The input list may contain records related to any topic/partition, for records of the same\n> topic/partition the order between them is guaranteed to be the same of the input, for records\n> of different topic/partition no order is guaranteed between them.\n>\n> See the partial error example below for more cotext.\n>","ref":"Klife.Client.html#c:produce_batch/2","title":"Klife.Client.produce_batch/2","type":"callback"},{"doc":"* `:producer` (`t:atom/0`) - Producer's name that will override the `default_producer` configuration. Ignored inside transactions.\n\n* `:partitioner` (`t:atom/0`) - Module that will override `default_partitioner` configuration.","ref":"Klife.Client.html#c:produce_batch/2-options","title":"Options - Klife.Client.produce_batch/2","type":"callback"},{"doc":"iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> [{:ok, _resp1}, {:ok, _resp2}, {:ok, _resp3}] = MyClient.produce_batch([rec1, rec2, rec3])\n\nPartial error example:\n\n    iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: :rand.bytes(2_000_000), topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> [{:ok, _resp1}, {:error, %Klife.Record{error_code: 10}}, {:ok, _resp3}] = MyClient.produce_batch([rec1, rec2, rec3])\n\nIn order to facilitate the response handling you can use `Klife.Record.verify_batch/1` or\n`Klife.Record.verify_batch!/1` functions.","ref":"Klife.Client.html#c:produce_batch/2-examples","title":"Examples - Klife.Client.produce_batch/2","type":"callback"},{"doc":"Produce a batch of records asynchronoulsy.\n\nThe same as [`produce_batch/2`](c:produce_batch/2) but returns immediately. Accepts a callback\noption to execute arbitrary code after response is obtained\n\n> #### Semantics and guarantees {: .info}\n> When callback is provided this functions is implemented as `Task.start/1`\n> calling [`produce_batch/2`](c:produce_batch/2) and executing the callback right after.\n> Therefore there is no guarantees about record delivery or callback execution.\n\n> #### Beware of process limits {: .warning}\n> Because this function spawns a new process for every new call with a callback defined\n> it may lead to a high number of processes to be spawned if it is executed inside loops.\n>\n> In order to avoid this you can increase the batch size you are calling it or\n> increase the system's process limit erlang flag.","ref":"Klife.Client.html#c:produce_batch_async/2","title":"Klife.Client.produce_batch_async/2","type":"callback"},{"doc":"* `:producer` (`t:atom/0`) - Producer's name that will override the `default_producer` configuration. Ignored inside transactions.\n\n* `:partitioner` (`t:atom/0`) - Module that will override `default_partitioner` configuration.\n\n* `:callback` (`t:term/0`) - MFA or function/1 that will be called with the produce result. The result is injected as the first argument on MFA and is the only argument for anonymous functions","ref":"Klife.Client.html#c:produce_batch_async/2-options","title":"Options - Klife.Client.produce_batch_async/2","type":"callback"},{"doc":"Anonymous Function:\n\n    iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> input = [rec1, rec2, rec3]\n    iex> :ok = MyClient.produce_batch_async(input, callback: fn resp ->\n    ...>  [{:ok, _resp1}, {:ok, _resp2}, {:ok, _resp3}] = resp\n    ...> end)\n\n    Using MFA:\n\n    iex> defmodule CB2 do\n    ...>    def exec(resp, my_arg1, my_arg2) do\n    ...>      \"arg1\" = my_arg1\n    ...>      \"arg2\" = my_arg2\n    ...>       [{:ok, _resp1}, {:ok, _resp2}, {:ok, _resp3}] = resp\n    ...>    end\n    ...> end\n    iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> input = [rec1, rec2, rec3]\n    iex> :ok = MyClient.produce_batch_async(input, callback: {CB2, :exec, [\"arg1\", \"arg2\"]})","ref":"Klife.Client.html#c:produce_batch_async/2-examples","title":"Examples - Klife.Client.produce_batch_async/2","type":"callback"},{"doc":"Transactionally produce a batch of records.\n\nIt expects a list of `Klife.Record` structs containg at least `:value` and `:topic` and returns\na tuple ok/error tuple along side with the enriched version of the input records as described\nin [\"Producer API Overview\"](m:Klife.Client#producer-api-overview).\n\nThe order of the response tuples on the returning list is the same as the input list. That means\nthe first response tuple will be related to the first record on the input and so on.\n\n> #### Beware of performance costs {: .warning}\n> Each `produce_batch_txn/2` will have 2 extra network roundtrips to the broker than a non\n> transactional `produce_batch/2`. One for adding topic/partitions to the transaction and other\n> to commit or abort it.","ref":"Klife.Client.html#c:produce_batch_txn/2","title":"Klife.Client.produce_batch_txn/2","type":"callback"},{"doc":"* `:pool_name` (`t:atom/0`) - Txn pool's name that will override the `default_txn_pool` configuration.","ref":"Klife.Client.html#c:produce_batch_txn/2-options","title":"Options - Klife.Client.produce_batch_txn/2","type":"callback"},{"doc":"iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> {:ok, [_resp1, _resp2, _resp3]} = MyClient.produce_batch_txn([rec1, rec2, rec3])","ref":"Klife.Client.html#c:produce_batch_txn/2-examples","title":"Examples - Klife.Client.produce_batch_txn/2","type":"callback"},{"doc":"Runs the given function inside a transaction.\n\nEvery produce API call made inside the given function will be part of a transaction\nthat will only commit if the returning value of fun is `:ok` or `{:ok, _any}`, any\nother return value will abort all records produced inside the given function.\n\n> #### Beware of performance costs {: .warning}\n> Each produce call inside the input function may have 1 extra network roundtrip to the broker\n> than a normal non transactional call.\n>\n> At the end of the transaction another round trip is needed in order to commit or abort\n> the transaction.\n\n\n> #### Produce semantics inside transaction {: .info}\n> All produce API calls keeps the same semantics as they have outside a transaction. This means\n> that records produced using `produce_batch/2` may still succeed/fail independently and a\n> `produce/2` call may still fail. Therefore it is user's responsability to verify and abort\n> the transaction if needed.","ref":"Klife.Client.html#c:transaction/2","title":"Klife.Client.transaction/2","type":"callback"},{"doc":"* `:pool_name` (`t:atom/0`) - Txn pool's name that will override the `default_txn_pool` configuration.","ref":"Klife.Client.html#c:transaction/2-options","title":"Options - Klife.Client.transaction/2","type":"callback"},{"doc":"iex> {:ok, [_resp1, _resp2, _resp3]} = MyClient.transaction(fn ->\n    ...>  rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    ...>  {:ok, resp1} = MyClient.produce(rec1)\n    ...>  rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    ...>  rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    ...>  [resp2, resp3] = MyClient.produce_batch([rec2, rec3])\n    ...>  {:ok, [resp1, resp2, resp3]}\n    ...> end)","ref":"Klife.Client.html#c:transaction/2-examples","title":"Examples - Klife.Client.transaction/2","type":"callback"},{"doc":"","ref":"Klife.Client.html#t:list_of_records/0","title":"Klife.Client.list_of_records/0","type":"type"},{"doc":"","ref":"Klife.Client.html#t:record/0","title":"Klife.Client.record/0","type":"type"},{"doc":"Kafka record representation.\n\nRepresents a Kafka record struct that will be used in the `Klife.Client` APIs.\n\nIn general terms it can be used to represent input or output data.\n\nAs an input the `Klife.Record` may have the following attributes:\n- `:value` (required)\n- `:topic` (required)\n- `:key` (optional)\n- `:headers` (optional)\n- `:partition` (optional)\n\nAs an output the input record will be added with one or more the following attributes:\n- `:offset` (if it was succesfully written)\n- `:partition` (if it was not present in the input)\n- `:error_code` (if something goes wrong on produce. See [kafka protocol error code](https://kafka.apache.org/11/protocol.html#protocol_error_codes) for context)","ref":"Klife.Record.html","title":"Klife.Record","type":"module"},{"doc":"","ref":"Klife.Record.html#t/0","title":"Klife.Record.t/0","type":"function"},{"doc":"Utility function to verify if all records in a `produce_batch/3` were successfully produced.","ref":"Klife.Record.html#verify_batch/1","title":"Klife.Record.verify_batch/1","type":"function"},{"doc":"iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> input = [rec1, rec2, rec3]\n    iex> {:ok, [%Klife.Record{value: \"my_val_1\"}, _r2, _r3]} = MyClient.produce_batch(input) |> Klife.Record.verify_batch()\n\nPartial error example. Notice that records 1 and 3 were successfully produced and only record 2\nhas errors, so the function will return `{:error, [rec1, rec2, rec3]}`\n\n    iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: :rand.bytes(2_000_000), topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> input = [rec1, rec2, rec3]\n    iex> {:error, [_rec1, %Klife.Record{error_code: 10}, _rec3]} = MyClient.produce_batch(input) |> Klife.Record.verify_batch()","ref":"Klife.Record.html#verify_batch/1-examples","title":"Examples - Klife.Record.verify_batch/1","type":"function"},{"doc":"Same as `verify_batch/1` but raises if any record fails and does not return ok/error tuple.","ref":"Klife.Record.html#verify_batch!/1","title":"Klife.Record.verify_batch!/1","type":"function"},{"doc":"iex> rec1 = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n    iex> rec2 = %Klife.Record{value: \"my_val_2\", topic: \"my_topic_2\"}\n    iex> rec3 = %Klife.Record{value: \"my_val_3\", topic: \"my_topic_3\"}\n    iex> input = [rec1, rec2, rec3]\n    iex> [_r1, _r2, _r3] = MyClient.produce_batch(input) |> Klife.Record.verify_batch!()","ref":"Klife.Record.html#verify_batch!/1-examples","title":"Examples - Klife.Record.verify_batch!/1","type":"function"},{"doc":"","ref":"Klife.Record.html#t:t/0","title":"Klife.Record.t/0","type":"type"},{"doc":"Defines a topic configuration.\n\nFor now this struct is only useful for initial client configuration, but in the future it may be useful for the admin api as well.","ref":"Klife.Topic.html","title":"Klife.Topic","type":"module"},{"doc":"* `:name` (`t:String.t/0`) - Required. Topic's name\n\n* `:default_producer` (`t:atom/0`) - Define the default producer to be used on produce API calls.\n\n* `:default_partitioner` (`t:atom/0`) - Define the default partitioner module to be used on produce API calls. Must implement `Klife.Behaviours.Partitioner`","ref":"Klife.Topic.html#module-client-configurations","title":"Client configurations - Klife.Topic","type":"module"},{"doc":"Defines a producer.","ref":"Klife.Producer.html","title":"Klife.Producer","type":"module"},{"doc":"* `:name` - Required. Producer name. Must be unique per client. Can be used as an option on the producer api\n\n* `:client_id` (`t:String.t/0`) - String used on all requests for the client. If not provided the following string is used: \"klife_producer.{client_name}.{producer_name}\"\n\n* `:acks` (`:all`, 1) - The number of broker's acks the producer requires before considering a request complete. `:all` means all ISR(in sync replicas) The default value is `:all`.\n\n* `:linger_ms` (`t:non_neg_integer/0`) - The maximum time to wait for additional messages before sending a batch to the broker. The default value is `0`.\n\n* `:batch_size_bytes` (`t:non_neg_integer/0`) - The maximum size of the batch of messages that the producer will send to the broker in a single request. The default value is `512000`.\n\n* `:delivery_timeout_ms` (`t:non_neg_integer/0`) - The maximum amount of time the producer will retry to deliver a message before timing out and failing the send. The default value is `60000`.\n\n* `:request_timeout_ms` (`t:non_neg_integer/0`) - The maximum amount of time the producer will wait for a broker response to a request before considering it as failed. The default value is `15000`.\n\n* `:retry_backoff_ms` (`t:non_neg_integer/0`) - The amount of time that the producer waits before retrying a failed request to the broker. The default value is `1000`.\n\n* `:max_in_flight_requests` (`t:non_neg_integer/0`) - The maximum number of unacknowledged requests per broker the producer will send before waiting for acknowledgments. The default value is `1`.\n\n* `:batchers_count` (`t:pos_integer/0`) - The number of batchers per broker the producer will start. See `Batchers Count` session for more details. The default value is `1`.\n\n* `:enable_idempotence` (`t:boolean/0`) - Indicates if the producer will use kafka idempotency capabilities for exactly once semantics. The default value is `true`.\n\n* `:compression_type` (`:none`, `:gzip` or `:snappy`) - The compression algorithm to be used for compressing messages before they are sent to the broker. The default value is `:none`.","ref":"Klife.Producer.html#module-client-configurations","title":"Client configurations - Klife.Producer","type":"module"},{"doc":"When configuring `Klife.Client`, users can specify a list of producers to be initialized for sending records to the Kafka cluster.\n\nOnce configured, users can interact with these producers through the `Klife.Client` producer API.","ref":"Klife.Producer.html#module-interacting-with-producers","title":"Interacting with producers - Klife.Producer","type":"module"},{"doc":"By default, all records are produced using a single default producer configured with standard settings, maximizing batch efficiency.\n\nThere are two main reasons to consider using multiple producers:\n\n- Different configurations for specific topics: Some topics may require unique settings.\nIn this case, you can assign topics with similar configuration needs to the same producer.\n\n- Fault isolation: Each producer has an independent queue of messages.\nUsing multiple producers can help isolate issues in one topic from affecting the performance of others.\n\nLet's dive in an example of fault isolation. Consider this scenario:\n\n```md\n- A producer batches messages for topics A and B into a single request to the broker leader.\n- For some reason, topic B is temporarily unavailable and fails, but topic A remains functional.\n- The producer is configured with in_flight_requests = 1 and delivery_timeout_ms = 1 minute.\n- This means the producer will retry sending records for topic B for up to one minute.\n- Because in_flight_requests is set to 1, all other records, including those for topic A, must wait\nuntil retries for topic B are exhausted.\n```\nIn this situation, issues with one topic can delay other topics handled by the same producer,\nas the retry mechanism occupies the single in-flight request slot.\n\nIf this scenario presents a potential issue for your use case, consider creating multiple producers\nand dedicating some of them to critical topics. However, note that this approach may slightly reduce\nbatch efficiency in normal operation to improve resilience in specific failure scenarios, which may\nbe infrequent.","ref":"Klife.Producer.html#module-how-many-producers","title":"How many producers? - Klife.Producer","type":"module"},{"doc":"Kafka guarantees message order only within the same topic and partition so this is the maximum\nlevel of ordering that the producer can provide as well.\n\nHowever, certain scenarios can lead to records being produced to Kafka out of order. In this context,\n\"out of order\" means the following:\n\n```markdown\n1. `rec1` and `rec2` are produced to the same topic and partition.\n2. The user calls produce(rec1) before calling produce(rec2).\n3. Both produce calls complete successfully.\n4. `rec1` is stored in the broker after `rec2`, resulting in `rec1.offset > rec2.offset`.\n```\n\nIn this case, rec1 and rec2 are considered \"out of order.\"\n\nThe ordering behavior for producers depends on their configuration:\n\n- Records produced by different producers (even if targeting the same topic and partition) may\nbe out of order since each producer operates independently and in parallel.\n- Records produced by a producer with `max_in_flight_requests` > 1 and `enable_idempotence` set to false\nmay be out of order due to network failures and retries.\n- Any records produced by a producer with `max_in_flight_requests` = 1 have guaranteed ordering.\n- Any records produced by a producer with `enable_idempotence` = true have guaranteed ordering.","ref":"Klife.Producer.html#module-order-guarantees","title":"Order guarantees - Klife.Producer","type":"module"},{"doc":"Klife’s producer uses dynamic batching, which automatically accumulates records that cannot be sent\nimmediately due to in_flight_request limitations. As a result, it is rarely necessary\nto set linger_ms to a value greater than zero.\n\nTypically, increasing `linger_ms` can improve batching efficiency, which benefits high-throughput topics.\nHowever, if your topic already has high throughput, dynamic batching will likely handle batching\neffectively without adjusting `linger_ms`.\n\nIncreasing linger_ms may be helpful only if you set a very high value for in_flight_request or\nif you need to limit request rates to the broker for specific reasons.","ref":"Klife.Producer.html#module-dynamic-batching","title":"Dynamic batching - Klife.Producer","type":"module"},{"doc":"Each Klife producer starts a configurable number of batchers for each broker in the Kafka cluster.\nTopics and partitions with the same leader are managed by the same batcher.\n\nBy default, Klife initializes only one batcher per broker, which optimizes batch efficiency but\nmay underutilize CPU resources on high-core systems.\n\nConsider the following setup:\n\n- A Kafka cluster with 3 brokers\n- An application running on a machine with 64 cores\n- `batchers_count` = 1\n\nIn this scenario, the application could encounter a performance bottleneck due to having only\nthree batchers handling all record production requests. Increasing parallelism by adjusting `batcher_count`\ncan help resolve this issue.\n\nFor instance, increasing batcher_count from 1 to 5 would create 15 batchers (5 per broker), potentially improving\nparallelism and CPU utilization.\n\n#### Some caveats:\n\n- The same topic and partition are always handled by the same batcher.\n- Higher batcher counts reduce batch efficiency, which may lower overall throughput.\n- The ideal setting may vary depending on your workload, so it's best to measure and\nadjust batcher_count based on your specific performance needs.","ref":"Klife.Producer.html#module-batchers-count","title":"Batchers Count - Klife.Producer","type":"module"},{"doc":"Each topic has a designated default producer, which is used by the producer client API\nfor regular produce calls. While this default producer can be overridden with options\nin the client API, be mindful that doing so may affect order guarantees.","ref":"Klife.Producer.html#module-topic-default-producer","title":"Topic default producer - Klife.Producer","type":"module"},{"doc":"If a produce call is made to a topic without a predefined default producer,\nthe client default producer is used. The client default producer can be configured as part\nof the overall `Klife.Client` setup. Refer to `Klife.Client` documentation for more details.","ref":"Klife.Producer.html#module-client-default-producer","title":"Client default producer - Klife.Producer","type":"module"},{"doc":"","ref":"Klife.Producer.html#produce_async/3","title":"Klife.Producer.produce_async/3","type":"function"},{"doc":"Default partitioner implementation.\n\nUses the following logic:\n\n- if record key is `nil` than defines a random partition\n- if record key is not `nil` than define a partition using phash2 function\n\nSource code is something like this:\n\n```elixir\ndefmodule Klife.Producer.DefaultPartitioner do\n  @behaviour Klife.Behaviours.Partitioner\n  alias Klife.Record\n\n  @impl Klife.Behaviours.Partitioner\n  def get_partition(%Record{key: nil}, max_partition),\n    do: :rand.uniform(max_partition + 1) - 1\n\n  def get_partition(%Record{key: key}, max_partition),\n    do: :erlang.phash2(key, max_partition + 1)\nend\n```","ref":"Klife.Producer.DefaultPartitioner.html","title":"Klife.Producer.DefaultPartitioner","type":"module"},{"doc":"Pool of transactional producers.","ref":"Klife.TxnProducerPool.html","title":"Klife.TxnProducerPool","type":"module"},{"doc":"* `:delivery_timeout_ms` (`t:non_neg_integer/0`) - The maximum amount of time the producer will retry to deliver a message before timing out and failing the send. The default value is `60000`.\n\n* `:request_timeout_ms` (`t:non_neg_integer/0`) - The maximum amount of time the producer will wait for a broker response to a request before considering it as failed. The default value is `15000`.\n\n* `:retry_backoff_ms` (`t:non_neg_integer/0`) - The amount of time that the producer waits before retrying a failed request to the broker. The default value is `1000`.\n\n* `:compression_type` (`:none`, `:gzip` or `:snappy`) - The compression algorithm to be used for compressing messages before they are sent to the broker. The default value is `:none`.\n\n* `:name` (`t:atom/0`) - Required. Pool name. Can be used as an option on the transactional api\n\n* `:base_txn_id` (`t:String.t/0`) - Prefix used to define the `transactional_id` for the transactional producers. If not provided, a random string will be used. The default value is `\"\"`.\n\n* `:pool_size` (`t:non_neg_integer/0`) - Number of transactional producers in the pool The default value is `20`.\n\n* `:txn_timeout_ms` (`t:non_neg_integer/0`) - The maximum amount of time, in milliseconds, that a transactional producer is allowed to remain open without either committing or aborting a transaction before it is considered expired The default value is `90000`.","ref":"Klife.TxnProducerPool.html#module-client-configurations","title":"Client configurations - Klife.TxnProducerPool","type":"module"},{"doc":"When configuring `Klife.Client`, users can specify a list of transactional pools to be\ninitialized for sending records to the Kafka cluster.\n\nOnce configured, users can interact with these transactional producers through\nthe `Klife.Client` transaction API.","ref":"Klife.TxnProducerPool.html#module-how-to-use","title":"How to use? - Klife.TxnProducerPool","type":"module"},{"doc":"Since a single producer can only have a single open transaction at any given time\nKlife starts a pool of transactional producers that can be checked out by other\nprocesses in order to execute transactions.\n\nEach transactional producer is a standard `Klife.Producer` but with transactional\ncapabilities. The transactional behaviour can be tweaked using its specific configurations.","ref":"Klife.TxnProducerPool.html#module-basic-understanding","title":"Basic understanding - Klife.TxnProducerPool","type":"module"},{"doc":"A more practical and guided discussion about transaction semantics can be found on `Klife.Client` transaction API.","ref":"Klife.TxnProducerPool.html#module-semantics","title":"Semantics - Klife.TxnProducerPool","type":"module"},{"doc":"Checks if the current process is under a Kafka transaction.","ref":"Klife.TxnProducerPool.html#in_txn?/1","title":"Klife.TxnProducerPool.in_txn?/1","type":"function"},{"doc":"iex> false = Klife.TxnProducerPool.in_txn?(MyClient)","ref":"Klife.TxnProducerPool.html#in_txn?/1-example","title":"Example - Klife.TxnProducerPool.in_txn?/1","type":"function"},{"doc":"Testing helper functions.\n\nIn order to test Kafka behaviour on tests we can have 2 approachs:\n\n- Having a running Kafka broker locally and testing against it\n- Mocking all external calls to the broker\n\n`Klife.Testing` supports the first approach by offering helper functions in order to\nverify if a record with the given list of properties exists in the broker.\n\nYou can use it like this:\n\n    # on test_helper.exs\n    Klife.Testing.setup(MyClient)\n\n    # on your test file\n    Klife.Testing.all_produced(MyClient, \"my_topic_a\", value: \"abc\")\n\nThe mocks approach is not supported directly by Klife but can be achieved using some\nawesome community libraries such as [Mimic](https://github.com/edgurgel/mimic) or\n[Mox](https://github.com/dashbitco/mox).","ref":"Klife.Testing.html","title":"Klife.Testing","type":"module"},{"doc":"Return a list of `Klife.Record` that match the given filters.\n\nYou can search by 3 fields:\n- value: binary\n- key: binary\n- headers: list of maps %{key: binary, value: binary}\n\nThe semantics between all possible fields is \"and\". Which means we only return records\nthat have match on all 3 filters if all 3 are available.\n\nThe semantics on the headers list is also \"and\". Which means we only return records\nthat have match with all headers given on the list.","ref":"Klife.Testing.html#all_produced/3","title":"Klife.Testing.all_produced/3","type":"function"},{"doc":"iex> val = :rand.bytes(1000)\n    iex> rec = %Klife.Record{value: val, topic: \"my_topic_1\"}\n    iex> {:ok, %Klife.Record{}} = MyClient.produce(rec)\n    iex> [%Klife.Record{}] = Klife.Testing.all_produced(MyClient, \"my_topic_1\", value: val)","ref":"Klife.Testing.html#all_produced/3-examples","title":"Examples - Klife.Testing.all_produced/3","type":"function"},{"doc":"","ref":"Klife.Testing.html#get_latest_offsets/3","title":"Klife.Testing.get_latest_offsets/3","type":"function"},{"doc":"Setup `Klife.Testing`, call it on your `test_helper.exs`.\n\nIn order to avoid big searchs on big local running Kafka topics, this setup retrieves\nall te current latests offsets and stores it to only search after them.","ref":"Klife.Testing.html#setup/1","title":"Klife.Testing.setup/1","type":"function"},{"doc":"Behaviour to define a partitioner\n\nModules must implement this behaviour in order to be used as partitioners on\nthe `Klife.Client` producer API.","ref":"Klife.Behaviours.Partitioner.html","title":"Klife.Behaviours.Partitioner","type":"behaviour"},{"doc":"Define the partition of a given record.\n\nReceives the record and the highest known partition for the record's topic.\n\nMust return an integer between 0 and `max_partition`. See `Klife.Producer.DefaultPartitioner` for\nan example.","ref":"Klife.Behaviours.Partitioner.html#c:get_partition/2","title":"Klife.Behaviours.Partitioner.get_partition/2","type":"callback"},{"doc":"Example test client","ref":"MyClient.html","title":"MyClient","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"MyClient.html#child_spec/1","title":"MyClient.child_spec/1","type":"function"},{"doc":"","ref":"MyClient.html#get_default_partitioner/0","title":"MyClient.get_default_partitioner/0","type":"function"},{"doc":"","ref":"MyClient.html#get_default_producer/0","title":"MyClient.get_default_producer/0","type":"function"},{"doc":"","ref":"MyClient.html#get_default_txn_pool/0","title":"MyClient.get_default_txn_pool/0","type":"function"},{"doc":"","ref":"MyClient.html#produce/2","title":"MyClient.produce/2","type":"function"},{"doc":"","ref":"MyClient.html#produce_async/2","title":"MyClient.produce_async/2","type":"function"},{"doc":"","ref":"MyClient.html#produce_batch/2","title":"MyClient.produce_batch/2","type":"function"},{"doc":"","ref":"MyClient.html#produce_batch_async/2","title":"MyClient.produce_batch_async/2","type":"function"},{"doc":"","ref":"MyClient.html#produce_batch_txn/2","title":"MyClient.produce_batch_txn/2","type":"function"},{"doc":"","ref":"MyClient.html#transaction/2","title":"MyClient.transaction/2","type":"function"},{"doc":"[![ci](https://github.com/oliveigah/klife/actions/workflows/ci.yml/badge.svg)](https://github.com/oliveigah/klife/actions/workflows/ci.yml)\n[![hex.pm badge](https://img.shields.io/badge/Package%20on%20hex.pm-informational)](https://hex.pm/packages/klife)\n[![Documentation badge](https://img.shields.io/badge/Documentation-ff69b4)](https://hexdocs.pm/klife)\n\n# Klife\n\nKlife is a high-performance Kafka client built from the ground up with minimal dependencies.\nCurrently, Klife supports producer functionality, with plans to add consumer features in the future.\n\nTo achieve high batch efficiency and ensure compatibility with evolving protocol versions, Klife\nleverages [Klife Protocol](https://github.com/oliveigah/klife_protocol). This efficiency allows\nKlife to deliver exceptional performance, with throughput improvements of up to 15x over other\ncommunity Kafka clients in some scenarios.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Currently, Klife provides producer functionality, with plans to expand into consumer\nfeatures as the project develops. Key features include:\n\n- **Efficient Batching**: Batches data to the same broker in a single TCP request per producer.\n- **Minimal Resource Usage**: Only one connection per broker for each client, optimizing resource usage.\n- **Exactly Once Semantics (EOS)**: Providing safe retries with idempotency on the protocol level.\n- **Synchronous and Asynchronous Produce Options**: Synchronous produces return the offset, while asynchronous produces support callbacks.\n- **Batch Produce API**: Allows batching for multiple topics and partitions.\n- **Automatic Cluster and Metadata Management**: Automatically adapts to changes in cluster topology and metadata.\n- **Testing Utilities**: Includes helper functions for testing against a real broker without complex mocking.\n- **Simple Configuration**: Streamlined setup for straightforward use.\n- **Comprehensive Documentation**: Includes examples and explanations of trade-offs.\n- **Custom Partitioner per Topic**: Configurable partitioning for each topic.\n- **Transactional Support**: Supports transactions in an Ecto-like style.\n- **SASL Authentication**: Currently supports plain authentication.\n- **Protocol Compatibility**: Supports recent protocol versions, with forward compatibility in mind.","ref":"readme.html#features","title":"Features - README","type":"extras"},{"doc":"","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:klife, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"readme.html#add-klife-to-your-list-of-dependencies-in-mix-exs","title":"Add `klife` to your list of dependencies in `mix.exs`: - README","type":"extras"},{"doc":"","ref":"readme.html#basic-usage","title":"Basic Usage - README","type":"extras"},{"doc":"```elixir\ndefmodule MyApp.Client do\n  use Klife.Client, otp_app: :my_app\nend\n```","ref":"readme.html#define-your-application-client","title":"Define your application client - README","type":"extras"},{"doc":"```elixir\nconfig :my_app, MyApp.Client,\n  connection: [\n    bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n    ssl: false\n  ]\n```","ref":"readme.html#add-basic-configuration","title":"Add basic configuration - README","type":"extras"},{"doc":"```elixir\nchildren = [ MyApp.Client ]\n\nopts = [strategy: :one_for_one, name: Example.Supervisor]\nSupervisor.start_link(children, opts)\n```","ref":"readme.html#add-the-client-to-the-supervision-tree","title":"Add the client to the supervision tree - README","type":"extras"},{"doc":"```elixir\nmy_rec = %Klife.Record{value: \"my_val_1\", topic: \"my_topic_1\"}\n{:ok, %Klife.Record} = MyApp.Client.produce(my_rec)\n```\n\nCheckout the `Klife.Client` [docs](https://hexdocs.pm/klife/Klife.Client.html) for more details","ref":"readme.html#call-the-producer-api","title":"Call the producer API - README","type":"extras"},{"doc":"I've test it against the 3 awesome community kafka libraries [brod](https://github.com/kafka4beam/brod)\nand [kafka_ex](https://github.com/kafkaex/kafka_ex) and [erlkaf](https://github.com/silviucpp/erlkaf)\nwhich are the most popular ones.\n\nThe relevant client configuration should be equal on all clients and they are:\n\n- required_acks: all\n- max_inflight_request: 1\n- linger_ms: 0\n- max_batch_size: 512kb","ref":"readme.html#producer-performance","title":"Producer performance - README","type":"extras"},{"doc":"In order to test sync produce performance we prepared a benchmark that uses [benchee](https://github.com/bencheeorg/benchee) to produce\nkafka records on kafka cluster running locally.\n\nThe details can be checked out on `benchmark.ex` mix task and the results on `bechmark_results`.\n\nTo reproduce it on your setup you can run (16 is the benchee parallel value):\n\n```\nbash start-kafka.sh\nmix benchmark producer_sync 16\n```\n\nEach iteration of the benchmark produces 3 records for 3 different topics in paralel and wait for the completion\nin order to move to the next iteration.\n\nThe main point driving the Klife's performance is the batching efficiency. As far as I can tell:\n\n- Klife: Batches everything that can be batched together in a single TCP request\n- Brod: Batches records only for the same topic/partition in a single TCP request\n- Kafka_ex: Does not batch records (I'm not sure if there is a way to change this behaviour)\n\nWith this scenario I've executed the benchmark increasing the `parallel` attribute from\nbenchee from 1 to 16, doubling it each round. The results are the following:\n\n![](./assets/producer_sync_benchmark.png \"Producer Sync Benchmark Results\")","ref":"readme.html#produce-sync","title":"Produce sync - README","type":"extras"},{"doc":"In order to test async produce performance we prepared a test script that produces records asynchronously on\na kafka cluster running locally.\n\nThe asynchronous benchmark spawns N parallel processes producing to one of 3 topics in a loop. After 10 seconds,\nit calculates the difference between the initial and current offsets for each topic partition to determine\nthe total records produced and the throughput (records per second).\n\nThe details can be checked out on `async_producer_benchmark.ex`.\n\nTo reproduce it on your setup you can run (16 is the N value):\n\n```\nbash start-kafka.sh\nmix benchmark producer_async 16\n```\n\n![](./assets/producer_async_benchmark.png \"Producer Async Benchmark Results\")","ref":"readme.html#produce-async","title":"Produce async - README","type":"extras"},{"doc":"Although `Klife Protocol` give us the capability to support all the latests versions\nfor now Klife uses fixed versions of the protocol that are not the latest for each message.\n\nI have plans to evolve this slowly as the project grows and I find a good way to deal\nwith multiple protocol versions at the same time on the code.\n\nFor now the message versions can be checked at `lib/klife/connection/message_versions.ex`\n\nFor performance reasons I'm aiming to support only versions after the flexible version that\nwere introduced on kafka 2.4 on KIP-482.\n\nBut should not be hard to support versions prior to that, if you are willing to try Klife\nbut you use an older version of kafka let me know and we can see if it is possible.","ref":"readme.html#compatibility-with-kafka-versions","title":"Compatibility with Kafka versions - README","type":"extras"},{"doc":"# Client configuration\n\nHere are some client configuration examples.","ref":"client_configuration.html","title":"Client configuration","type":"extras"},{"doc":"```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: false\n    ]\n```\n\nThis client will connect to brokers using non ssl connection and produce messages using the default producer and default partitioner.","ref":"client_configuration.html#simplest-configuration","title":"Simplest configuration - Client configuration","type":"extras"},{"doc":"```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: true,\n      connect_opts: [\n        verify: :verify_peer,\n        cacertfile: Path.relative(\"test/compose_files/ssl/ca.crt\")\n      ],\n      socket_opts: [delay_send: true],\n      sasl_opts: [\n        mechanism: \"PLAIN\",\n        mechanism_opts: [\n          username: \"klifeusr\",\n          password: \"klifepwd\"\n        ]\n      ],\n    ]\n```\n\nThis client will connect to brokers using ssl connection, `connect_opts` and `socket_opts` are forwarded to erlang module `:ssl` in order to proper configure the socket. See the documentation for more details.","ref":"client_configuration.html#ssl-with-sasl-and-custom-socket-opts","title":"SSL with SASL and custom socket opts - Client configuration","type":"extras"},{"doc":"```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: false\n    ],\n    producers: [\n      [\n        name: :my_linger_ms_producer,\n        linger_ms: 1_000\n      ],\n      [\n        name: :my_custom_client_id_producer,\n        client_id: \"my_custom_client_id\",\n      ]\n    ],\n    topics: [\n      [\n        name: \"my_topic_0\",\n        default_producer: :my_linger_ms_producer\n      ],\n      [\n        name: \"my_topic_1\",\n        default_producer: :my_custom_client_id_producer\n      ]\n    ]\n```\n\nThis client will have a total of 3 producers, the default one plus the other 2 defined in the configuration. You can see all the configuration options for the producers in `Klife.Producer`.\n\nMessages produced to `my_topic_0` and `my_topic_1` will use `my_linger_ms_producer` and `my_custom_client_id_producer` respectively if no producer is set on opts. All other topics keep using the default producer.","ref":"client_configuration.html#defining-and-using-multiple-producers","title":"Defining and using multiple producers - Client configuration","type":"extras"},{"doc":"First you need to implement a module following the `Klife.Behaviours.Partitioner` behaviour.\n\n```elixir\ndefmodule MyApp.MyCustomPartitioner do\n  @behaviour Klife.Behaviours.Partitioner\n\n  alias Klife.Record\n\n  @impl true\n  def get_partition(%Record{} = record, max_partition) do\n    # Some logic to find the partition here!\n  end\nend\n\n```\n\nThen, you need to use it on your configuration.\n\n```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: false\n    ],\n    topics: [\n      [\n        name: \"my_topic_0\",\n        default_partitioner: MyApp.MyCustomPartitioner\n      ]\n    ]\n```\n\nOn this client, the records produced to `my_topic_0` without a specific partition will have a partition assigned using the `MyApp.MyCustomPartitioner` module all other topics keep using the default partitioner.","ref":"client_configuration.html#defining-and-using-custom-partitioner","title":"Defining and using custom partitioner - Client configuration","type":"extras"},{"doc":"```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: false\n    ],\n    default_txn_pool: :my_txn_pool,\n    txn_pools: [\n      [name: :my_txn_pool, base_txn_id: \"my_custom_base_txn_id\"],\n      [name: :my_txn_pool_2, txn_timeout_ms: :timer.seconds(120)]\n    ]\n    topics: [[name: \"my_topic_0\"]]\n```\n\nThis client will have a total of 3 txn pools, the default one plus the other two defined in the configuration. You can see all the configuration options for the producers in `Klife.TxnProducerPool`.","ref":"client_configuration.html#defining-multiple-transactional-txn-pools","title":"Defining multiple transactional (txn) pools - Client configuration","type":"extras"},{"doc":"```elixir\n  config :my_app, MyApp.Client,\n    connection: [\n      bootstrap_servers: [\"localhost:19092\", \"localhost:29092\"],\n      ssl: false\n    ],\n    default_producer: :my_custom_producer,\n    producers: [[name: :my_custom_producer, linger_ms: 1_000]],\n    default_partitioner: MyCustomPartitioner,\n    default_txn_pool: :my_txn_pool,\n    txn_pools: [[name: :my_txn_pool, base_txn_id: \"my_custom_base_txn_id\"]],\n    topics: [[name: \"my_topic_0\"]]\n```\n\nThis cliente will have only one producer (`:my_custom_producer`) and txn pool (`:my_txn_pool`),and the default paritioner strategy will be `MyCustomPartitioner`. All this 3 configurations will be used in produce API calls to topics that does not have any override config defined in the `topics` configuration.","ref":"client_configuration.html#using-custom-default-producer-partitioner-and-txn-pool","title":"Using custom default producer, partitioner and txn pool - Client configuration","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}